@@ -22,6 +22,8 @@ use Cwd;
 
 use File::Copy::Recursive qw(dircopy);
 
+use File::Find;
+
 our @EXPORT = qw{
 
     search_plugins
@@ -95,7 +97,6 @@ sub show_installed_plugins {
 sub install_plugin_deps {
 
     my $plg_src = shift;
-    my %opts = @_;
 
     print "install deps for $plg_src ...\n";
 
@@ -109,7 +110,7 @@ sub install_plugin_deps {
       execute_shell_command("cd $plg_src && bundle --path local");
     }
 
-    if ( -f "$plg_src/requirements.txt" and ! $opts{skip_pip_requirements}){
+    if ( -f "$plg_src/requirements.txt"){
       open F, "$plg_src/sparrow.json" or confess "can't open file $plg_src/sparrow.json to read: $!";
       my $sp = join "", <F>;
       my $spj = decode_json($sp);
@@ -122,11 +123,54 @@ sub install_plugin_deps {
 
 }
 
+sub install_plugin_recursive {
+
+  my $path = shift or die "usage: install_plugin_recursive(\$path)";
+  my $force = shift;
+
+  die "directory [$path] does not exit" unless -d $path;
+
+  find(
+    { 
+      wanted => sub {
+
+        my $file = $_;
+      
+        return unless $file eq 'sparrow.json';
+      
+        my @args = (".","--local");
+      
+        push @args, "--force" if $force;
+      
+        install_plugin(@args);
+
+      }
+    }, $path
+  );  
+
+
+}
+
 sub install_plugin {
 
     my $pid  = shift or confess 'usage: install_plugin(name,opts)';
 
-    my %opts = @_;
+    my @args = @_;
+
+    my $version;
+    my $recursive;
+    my $local_install;
+    my $install_deps;
+    my $force;
+
+    my $args_st = GetOptionsFromArray(
+        \@args,
+        "local"             => \$local_install,
+        "force"             => \$force,
+        "install-deps"      => \$install_deps,
+        "recursive"         => \$recursive,
+        "version=s"         => \$version,
+    );
 
     my $ptype;
 
@@ -137,10 +181,15 @@ sub install_plugin {
 
     my $list = read_plugin_list('as_hash');
 
-    if ($pid eq '.'){ # install plugin from local source as public plugin
+    if ($recursive and $local_install){ # install plugin from local source recursively 
 
-      my $dir = getcwd;
+      install_plugin_recursive($pid, $force);
 
+    } elsif ( $local_install ) {  # install plugin from local source as a public plugin
+
+      my $dir = $pid eq '.'  ? getcwd : $pid;
+
+      print "install from source: $dir ... \n";	
       open F, "$dir/sparrow.json" or confess "can't open file $dir/sparrow.json to read: $!";
       my $sp = join "", <F>;
       my $spj = decode_json($sp);
@@ -151,6 +200,21 @@ sub install_plugin {
 
       print "install public\@$pid version $v from local source\n";
 
+      if (-f sparrow_root."/plugins/public/$pid/sparrow.json" ){
+  
+        open F, sparrow_root."/plugins/public/$pid/sparrow.json" or confess "can't open file to read: $!";
+        my $sp = join "", <F>;
+        my $spj = decode_json($sp);
+        close F;
+  
+        my $inst_v  = version->parse($spj->{version}||'0.0.0');
+  
+        if ($inst_v >= $v and ! $force ){
+            print "plugin is already istalled and has higher version: $inst_v\n";
+            return;
+        }
+  
+      }
 
       if ( -d sparrow_root()."/plugins/public/$pid" ){
         rmtree(sparrow_root()."/plugins/public/$pid") or die "can't remove dir: ".sparrow_root()."/plugins/public/$pid, error: $!";
@@ -174,7 +238,7 @@ sub install_plugin {
 
     } elsif ( $list->{'public@'.$pid} and $ptype ne 'private' ) {
 
-        if (! $opts{'--version'}  and  -f sparrow_root."/plugins/public/$pid/sparrow.json" ){
+        if (! $version  and  -f sparrow_root."/plugins/public/$pid/sparrow.json" ){
   
             open F, sparrow_root."/plugins/public/$pid/sparrow.json" or confess "can't open file to read: $!";
             my $sp = join "", <F>;
@@ -210,13 +274,12 @@ sub install_plugin {
 
                 print "public\@$pid is uptodate ($inst_v)\n";
 
-                install_plugin_deps(sparrow_root."/plugins/public/$pid", skip_pip_requirements => 1);
-
+                install_plugin_deps(sparrow_root."/plugins/public/$pid") if $install_deps;
             }
 
         } else {
 
-            my $v = $opts{'--version'} ||  $list->{'public@'.$pid}->{version};
+            my $v = $version ||  $list->{'public@'.$pid}->{version};
             my $vn = version->parse($v)->numify; 
             
             print "installing public\@$pid version $v ...\n";
@@ -243,30 +306,32 @@ sub install_plugin {
           
     } elsif ($list->{'private@'.$pid} and $ptype ne 'public' ) {
 
-        print "installing private\@$pid ...\n";
-
-        if ( -d sparrow_root."/plugins/private/$pid" ) {
-
-            execute_shell_command("cd ".sparrow_root."/plugins/private/$pid && git pull");
-            execute_shell_command("cd ".sparrow_root."/plugins/private/$pid && git config credential.helper 'cache --timeout=3000000'");
-
-            install_plugin_deps(sparrow_root."/plugins/private/$pid");
+        if ( -d sparrow_root()."/plugins/private/$pid/" ) {
 
+          print "installing private\@$pid ...\n";
+  
+          execute_shell_command("cd ".sparrow_root."/plugins/private/$pid && git pull");
+  
+          execute_shell_command("cd ".sparrow_root."/plugins/private/$pid && git config credential.helper 'cache --timeout=3000000'");
+  
+          install_plugin_deps(sparrow_root."/plugins/private/$pid");
+  
         } else {
 
-            execute_shell_command("git clone  ".($list->{'private@'.$pid}->{url}).' '.sparrow_root."/plugins/private/$pid");
-            execute_shell_command("cd ".sparrow_root."/plugins/private/$pid && git config credential.helper 'cache --timeout=3000000'");                
-
-            install_plugin_deps(sparrow_root."/plugins/private/$pid");
-
-      }
-
+          execute_shell_command("git clone  ".($list->{'private@'.$pid}->{url}).' '.sparrow_root."/plugins/private/$pid");
+    
+          execute_shell_command("cd ".sparrow_root."/plugins/private/$pid && git config credential.helper 'cache --timeout=3000000'");                
+    
+          install_plugin_deps(sparrow_root."/plugins/private/$pid");
+        }
     } elsif ( -d sparrow_root()."/plugins/public/$pid/" ) {
 
       print "plugin ".sparrow_root()."/plugins/public/$pid/ installed locally, nothing to do here ...\n";
 
     } else {
-        confess "unknown plugin";
+
+      confess "unknown plugin";
+
     }
 
 }
