@@ -238,9 +238,11 @@ sub remote_task_install {
 
         my $owner = $1;
 
+        my $unsecure_flag = $ENV{SPARROW_UNSECURE} ? "-k" : "";
+
         execute_shell_command(
             "mkdir -p ".(sparrow_root())."/cache/meta/$owner\@$project && ".
-            "curl -f -s ". sparrow_hub_api_url().'/api/v1/remote-task/meta/'.
+            "curl $unsecure_flag -f -s ". sparrow_hub_api_url().'/api/v1/remote-task/meta/'.
             "$owner/$project/$task -o $out_path",
             silent => 1 ,
         );
